package com.puppy.uhfexample.util;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.widget.Toast;


import java.io.IOException;
import java.lang.ref.WeakReference;
import java.util.Set;
import java.util.UUID;

import androidx.activity.result.ActivityResultCallback;
import androidx.activity.result.contract.ActivityResultContract;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;


public class BluetoothUtil {

    private BluetoothAdapter mBluetoothAdapter;
    private AppCompatActivity activity;
    private MyBroadcastReceiver myBroadcastReceiver;
    private BlueToothDeviceListener blueToothDeviceListener;
    private BluetoothSocket mBlueToothSocket;
    private BlueToothSocketListener mSocketListener;

    public BluetoothUtil(AppCompatActivity activity) {
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (mBluetoothAdapter == null) {
            Toast.makeText(activity.getApplicationContext(), "设备不支持蓝牙", Toast.LENGTH_SHORT).show();
            return;
        }
        this.activity = activity;
        if (myBroadcastReceiver == null) {
            myBroadcastReceiver = new MyBroadcastReceiver(this);
            IntentFilter filter = new IntentFilter();
            filter.addAction(BluetoothDevice.ACTION_FOUND);//获得扫描结果
            filter.addAction(BluetoothDevice.ACTION_BOND_STATE_CHANGED);//配对状态变化
            activity.registerReceiver(myBroadcastReceiver, filter);
        }
    }

    public BluetoothAdapter getBluetoothAdapter() {
        return mBluetoothAdapter;
    }

    /**
     * 请求打开蓝牙
     *
     * @param contract
     * @param callback
     */
    public BluetoothUtil requestOpenBlueTooth(@NonNull ActivityResultContract contract,
                                              @NonNull ActivityResultCallback callback) {
        if (!ifBlueToothEnabled()) {
            activity.registerForActivityResult(contract, callback).launch(new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE));
        }
        return this;
    }

    /**
     * 蓝牙是否开启
     *
     * @return
     */
    public boolean ifBlueToothEnabled() {
        return mBluetoothAdapter.isEnabled();
    }

    /**
     * 开始查找所有设备，包含已绑定设备
     *
     * @param blueToothDeviceListener
     * @return
     */
    public BluetoothUtil findAllBlueToothDevice(BlueToothDeviceListener blueToothDeviceListener) {
        this.blueToothDeviceListener = blueToothDeviceListener;
        if (getBondDevice() != null)
            blueToothDeviceListener.getBondedDevice(getBondDevice());
        if (!mBluetoothAdapter.isDiscovering())
            MLog.e(" blue discovery = " + mBluetoothAdapter.startDiscovery());
        return this;
    }


    /**
     * 停止查找所有设备
     *
     * @return
     */
    public BluetoothUtil stopFindBlueToothDevice() {
        if (mBluetoothAdapter.isDiscovering())
            mBluetoothAdapter.cancelDiscovery();
        return this;
    }

    /**
     * 获取已绑定的设备
     *
     * @return
     */
    public Set<BluetoothDevice> getBondDevice() {
        return mBluetoothAdapter.getBondedDevices();
    }

    /**
     * 和已配对设备创建蓝牙连接
     *
     * @param macAddress
     * @return
     */
    public BluetoothUtil createBluetoothSocket(String macAddress, BlueToothSocketListener mSocketListner) {
        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(macAddress);
        return createBluetoothSocket(device, mSocketListner);
    }

    public BluetoothUtil createBluetoothSocket(BluetoothDevice device, BlueToothSocketListener mSocketListener) {
        this.mSocketListener = mSocketListener;
        //未绑定的设备去绑定，然后去广播里面监听绑定状态后创建socket连接
        if (!getBondDevice().contains(device)) {
            if (!device.createBond()) {
                mSocketListener.getBluetoothSocket(null, MConstant.BLUETOOTH_BONDED_FAILED);
            }
        } else {
            //已绑定，直接建立socket通信即可
            createSocketConnect(device);
        }
        return this;
    }

    private void createSocketConnect(BluetoothDevice device) {
        // 蓝牙串口服务对应的UUID。如使用的是其它蓝牙服务，需更改下面的字符串
        int errorCode = MConstant.BLUETOOTH_SOCKET_CONNECT_SUCCESS;
        try {
            UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
            mBlueToothSocket = device.createRfcommSocketToServiceRecord(MY_UUID);
            mBlueToothSocket.connect();
        } catch (IOException e) {
            errorCode = MConstant.BLUETOOTH_SOCKET_CONNECT_FAILED;
            e.printStackTrace();
        } finally {
            mSocketListener.getBluetoothSocket(mBlueToothSocket, errorCode);
        }
    }


    public BluetoothSocket getBlueToothSocket() {
        return mBlueToothSocket;
    }

    public boolean ifBlueSocketConnected() {
        if (mBlueToothSocket != null)
            return mBlueToothSocket.isConnected();
        return false;
    }


    /**
     * 释放蓝牙资源和引用，防止内存泄漏以及蓝牙资源损耗
     */
    public void releaseResource() {
        stopFindBlueToothDevice();
        if (myBroadcastReceiver != null) {
            activity.unregisterReceiver(myBroadcastReceiver);
            myBroadcastReceiver = null;
        }
        activity = null;
        blueToothDeviceListener = null;
        mSocketListener = null;
    }


    static class MyBroadcastReceiver extends BroadcastReceiver {

        private WeakReference<BluetoothUtil> mWeak;

        public MyBroadcastReceiver(BluetoothUtil bluetoothUtil) {
            mWeak = new WeakReference<>(bluetoothUtil);
        }

        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            BluetoothDevice devices = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
            switch (action) {
                case BluetoothDevice.ACTION_FOUND:
                    BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                    if (mWeak != null && mWeak.get() != null) {
                        mWeak.get().blueToothDeviceListener.getNewBlueDevices(device);
                    }
                    break;
                case BluetoothDevice.ACTION_BOND_STATE_CHANGED:
                    int status = devices.getBondState();
                    if (status == BluetoothDevice.BOND_BONDED) {
                        if (mWeak != null && mWeak.get() != null) {
                            mWeak.get().createSocketConnect(devices);
                        }
                    }
                    break;
                default:
                    break;
            }

        }
    }


}
