package com.puppy.uhfexample.util;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Environment;
import android.provider.Settings;

import java.util.HashMap;
import java.util.Map;

import androidx.activity.result.ActivityResultCallback;
import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContract;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;

import static android.content.Context.LOCATION_SERVICE;

public class RequestOpenUtil {

    public static final byte OPEN_BLUETOOTH = 1;
    public static final byte REQUEST_FILE_PERMISSION = 2;
    public static final byte OPEN_GPS = 3;

    private static Map<Byte, ActivityResultLauncher<Intent>> activityResultLauncherList = new HashMap<>();

    public interface GetPermissionResultCode {
        void permissionResultCode(int resultCode);
    }

    /**
     * 此权限使用 {@link androidx.activity.ComponentActivity#registerForActivityResult(ActivityResultContract, ActivityResultCallback)}
     * ，只能resumed之前就调用，所以建议优先调用
     *
     * @param activity
     * @return
     */
    public static ActivityResultLauncher requestOpenBlueTooth(AppCompatActivity activity, GetPermissionResultCode getPermissionResultCode) {
        ActivityResultLauncher<Intent> activityResultLauncher = activity.registerForActivityResult(new ActivityResultContracts.StartActivityForResult(),
                result -> {
                    MLog.e(" requestOpenBlueTooth comet to here");
                    getPermissionResultCode.permissionResultCode(result.getResultCode());
                    launchAll(activity);
                });
        activityResultLauncherList.put(OPEN_BLUETOOTH, activityResultLauncher);
        return activityResultLauncher;
    }

    /**
     * 请求文件权限(包含SD文件权限)
     * 此权限需要跳转其它界面的权限，且使用 {@link androidx.activity.ComponentActivity#registerForActivityResult(ActivityResultContract, ActivityResultCallback)}
     * ，只能resumed之前就调用，所以建议优先调用
     *
     * @param activity 申请权限的activity
     */
    @RequiresApi(api = 30)
    public static ActivityResultLauncher requestOpenFilePermission(AppCompatActivity activity, GetPermissionResultCode getPermissionResultCode) {
        //判断是否给予了文件访问权限
        if (Environment.isExternalStorageManager()) {
            getPermissionResultCode.permissionResultCode(Activity.RESULT_OK);
            return null;
        }
        ActivityResultLauncher<Intent> activityResultLauncher = activity.registerForActivityResult(new ActivityResultContracts.StartActivityForResult(),
                result -> {
                    getPermissionResultCode.permissionResultCode(Environment.isExternalStorageManager() ? Activity.RESULT_OK : Activity.RESULT_CANCELED);
                    launchAll(activity);
                });
        activityResultLauncherList.put(REQUEST_FILE_PERMISSION, activityResultLauncher);
        return activityResultLauncher;
    }

    public static boolean ifGpsOpen(AppCompatActivity activity) {
        LocationManager manager = (LocationManager) activity.getSystemService(LOCATION_SERVICE);
        return manager.isProviderEnabled(LocationManager.GPS_PROVIDER);
    }

    /**
     * 请求开启GPS开关
     *
     * @param activity                当前请求的申请权限的activity
     * @param getPermissionResultCode 请求回调参数
     * @return ActivityResultLauncher示例
     */
    public static ActivityResultLauncher openLocationService(AppCompatActivity activity, GetPermissionResultCode getPermissionResultCode) {
        LocationManager manager = (LocationManager) activity.getSystemService(LOCATION_SERVICE);
        boolean gpsEnabled = manager.isProviderEnabled(LocationManager.GPS_PROVIDER);
        if (gpsEnabled) {
            getPermissionResultCode.permissionResultCode(Activity.RESULT_OK);
            return null;
        }
        boolean isNetwork = manager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
        if (!isNetwork) {
            ActivityResultLauncher<Intent> activityResultLauncher = activity.registerForActivityResult(new ActivityResultContracts.StartActivityForResult(),
                    result -> {
                        getPermissionResultCode.permissionResultCode(manager.isProviderEnabled(LocationManager.GPS_PROVIDER) ? Activity.RESULT_OK : Activity.RESULT_CANCELED);
                        launchAll(activity);
                    });
            activityResultLauncherList.put(OPEN_GPS, activityResultLauncher);
            return activityResultLauncher;
        } else {
            getPermissionResultCode.permissionResultCode(Activity.RESULT_OK);
        }
        return null;
    }


    /**
     * 启动所有请求过的权限，依次来请求
     *
     * @param context 当前activity的上下文对象
     */
    public static void launchAll(Context context) {
        for (Map.Entry<Byte, ActivityResultLauncher<Intent>> byteActivityResultLauncherEntry : activityResultLauncherList.entrySet()) {
            ActivityResultLauncher<Intent> mLauncher = byteActivityResultLauncherEntry.getValue();
//            MLog.e("key = " + byteActivityResultLauncherEntry.getKey());
            switch (byteActivityResultLauncherEntry.getKey()) {
                case OPEN_BLUETOOTH:
                    mLauncher.launch(new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE));
                    activityResultLauncherList.remove(OPEN_BLUETOOTH);
                    return;
                case REQUEST_FILE_PERMISSION:
                    if (android.os.Build.VERSION.SDK_INT >= 30) {
                        Intent intent = new Intent(Settings.ACTION_MANAGE_APP_ALL_FILES_ACCESS_PERMISSION);
                        intent.addCategory("android.intent.category.DEFAULT");
                        intent.setData(Uri.parse(String.format("package:%s", context.getPackageName())));
                        mLauncher.launch(intent);
                        activityResultLauncherList.remove(REQUEST_FILE_PERMISSION);
                    }
                    return;
                case OPEN_GPS:
                    mLauncher.launch(new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS));
                    activityResultLauncherList.remove(OPEN_GPS);
                    return;
                default:
                    break;
            }
        }
    }

}
